use strict;
use Module::Build;

Module::Build->new(
    module_name => "Module::Depends",
    license     => 'perl',
    build_requires => {
        'Test::More' => 0,
    },
    requires => {
        'YAML' => 0,
        'File::Spec'  => 0,
        'Class::Accessor::Chained' => 0,
    },
    create_makefile_pl => 'traditional',
   )
  ->create_build_script;
